<?php 

if(!defined('ABSPATH')) exit;

/**
 * Method class
 */
class Firstdata_Gateway extends WC_Payment_Gateway {
	
	public function __construct() {
		$this->id = 'firstdata';
		$this->has_fields = false;
		$this->method_title = __('Polcard Gateway', 'firstdata');
		$this->method_description = __('Polcard Gateway', 'firstdata');
		$this->init_form_fields();
		$this->init_settings();
		$this->init();
		$this->supports = array('subscriptions');
	}
	
	/**
	 * Initialize hooks
	 */
	public function init() {
	    $this->title = $this->get_option('title');
	    $this->description = $this->get_option('description');
	    if(!$this->title) $this->title = $this->method_title;
	    if(!$this->description) $this->description = $this->method_description;
	    add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ));
	    add_action( 'woocommerce_api_firstdata_gateway', array($this, 'api_cb'));
	}

	/**
	 * Initialize form for method settings (a tab in WC payement settings)
	 */
	public function init_form_fields() {
	    $this->form_fields = array(
	        'enabled' => array(
	            'title' => __( 'Enable/Disable', 'firstdata' ),
	            'type' => 'checkbox',
	            'label' => __( 'Enable Polcard Gateway', 'firstdata' ),
	            'default' => 'yes'
	        ),
	        'title' => array(
	            'title' => __( 'Title', 'firstdata' ),
	            'type' => 'text',
	            'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
	            'default' => __('Pay by Polcard Gateway (card/Blik/transfer etc.)', 'firstdata'),
	            'desc_tip'      => true,
	        ),
	        'description' => array(
	            'title' => __( 'Customer Message', 'firstdata' ),
	            'type' => 'textarea',
	            'default' => ''
	        )
	    );
	}

	/**
	 * Do a payment procedure for a new order
	 */
	function process_payment($order_id) {
	    $order = new WC_Order($order_id);

	    $currency = $order->get_currency();
	    if($currency != 'PLN') {
//	        error_log('Waluta niedozwolona: ' . $currency);
	        wc_add_notice( __('Currency not allowed for this payment method', 'firstdata'), 'error' );
	        return;
	    }
	    
//	    error_log('get='.print_r($_GET, true));
//	    error_log('post='.print_r($_GET, true));
	    
	    $args = array(
            'a' => 'r',
            'id' => $order_id,
	        't' => hash('sha256', $order_id . get_option('firstdata_key'))
        );
        return array(
	        'result' => 'success',
	        'redirect' => add_query_arg($args , WC()->api_request_url('Firstdata_Gateway'))
        );
	}
	
	/**
	 * Dispatcher for processing callback requests
	 */
	public function api_cb() {
	    $action = '';
//	    error_log('api cb get=' . print_r($_GET, true) . ' post=' . print_r($_POST, true));
	    if(isset($_GET['a'])) $action = $_GET['a'];
	    switch ($action) {
	        case 'r':
	            if(isset($_GET['id']) && isset($_GET['t']) && ($_GET['t'] == hash('sha256', $_GET['id'] . get_option('firstdata_key')))) {
	                $this->createRedir($_GET['id']);
    	            exit;
	            }
	            else {
//	                error_log('cb err E-20');
    	            wp_die(__('Invalid request','firstdata') . ' [E-20]');
	            }
	        case 'i':
	            $data = Firstdata_Api::validIpn();
	            if(gettype($data) == 'string') {
//	                error_log('cb err ' . $data);
	                wp_die(__('Invalid request','firstdata') . ' [' . $data . ']');
	            }
	            else {
	                $this->doReq($data);
	                exit;
	            }
	        case 'd':
	            $data = Firstdata_Api::validResp();
	            if(gettype($data) == 'string') {
//	                error_log('cb err ' . $data);
	                wp_die(__('Invalid request','firstdata') . ' [' . $data . ']');
	            }
	            else {
	               $this->doReq($data, true);
                   exit;
                }
	    }
	    wp_die(__('Invalid request','firstdata') . ' [E-01]');
	}

	/**
	 * Create redirection to FDP form
	 */
	function createRedir($order_id) {
	    $order = new WC_Order($order_id);
	    Firstdata_Api::doRedir($order);
	}

	/**
	 * Refresh order state after payment
	 */
	function doReq($data, $redir = false) {
	    $chargetotal = 1 * str_replace(',', '.', $_POST['chargetotal']);
	    
	    $tmp = explode('_', $_POST['oid']);
	    $order_id = $tmp[1];
	    $order = new WC_Order($order_id);
	    
	    $acode = explode(':', $_POST['approval_code']);
	    $status = $acode[0];
	    
	    $total = $order->get_total();
	    if($chargetotal != $total) {
	        $order->update_status('error', __('Wrong payment amount', 'firstdata'));
	        
	        $text = "Wrong payment amount.\r\n  order id: ".$_POST['oid'];
	        if(isset($data['ipgTransactionId']))
	            $text .= "\r\n  Transadtion id: ".$data['ipgTransactionId'];
	            error_log($text);

	            return false;
	    }
	    
	    if($status == 'N') {
	        $text = "Payment error.\r\n  error code: " . $acode[1] . "\r\n  msg: " . $acode[2] . "\r\n  order id: " . $_POST['oid'];
	        if(isset($_POST['ipgTransactionId']))
	            $text .= "\r\n  Transadtion id: ".$_POST['ipgTransactionId'];
	            error_log($text);
	            $this->updateTrans($order, $status, $chargetotal);
	            if($redir) wp_redirect($this->get_return_url($order));
	            
	            return false;
	    }
	    
	    if(!$this->updateTrans($order, $status, $chargetotal)) return false;
	    
	    $text = "Transaction complete\r\n  Transadtion id: ". $_POST['ipgTransactionId'];
	    error_log($text);
	    
	    if($redir) wp_redirect($this->get_return_url($order));
	    return true;
	}
			
	/**
	 * Save payment data in WC objects
	 */
	private function updateTrans($order, $status, $amount) {
	    $transaction_id = $_POST['ipgTransactionId'];
	    $acode = explode(':', $_POST['approval_code']);
	    
	    update_post_meta($order->get_id(), 'Payment type', $this->method_title);
	    
	    if(isset($_POST['cardnumber'])) update_post_meta($order->get_id(), 'Card number', $_POST['cardnumber']);
	    if(isset($_POST['ccbrand'])) update_post_meta($order->get_id(), 'Card brand', $_POST['ccbrand']);
	    if(isset($_POST['expmonth'])) update_post_meta($order->get_id(), 'Card number', $_POST['expmonth'] . '/' . $_POST['expyear']);
	    if(isset($_POST['card_holder'])) update_post_meta($order->get_id(), 'Card holder', $_POST['card_holder']);
	    
	    if($status == 'Y') {
	        if (!$order->has_status( array('processing', 'completed'))) {
//	            $order->add_order_note( $note );
	            $order->payment_complete($transaction_id);
	            WC()->cart->empty_cart();
	        }
	    }
	    elseif($status == '?') {
	        $order->update_status('on-hold');
	        WC()->cart->empty_cart();
	    }
	    else {
	        $order->update_status('failed', __('Payment error:', 'firstdata') . ' [' . $acode[1] . '] ' . $acode[2]);
	    }
	    return true;
	}
	
}

?>